library xwidgetsbase;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class XTextFormFieldLogin extends StatelessWidget {
  final String? label;
  final String? hintText;
  final String? initialValue;
  final IconData? icon;
  // final int maxLines = 1;
  //final int minLines;
  final Decoration? decoration;
  final ValueChanged<String>? onChanged;
  final TextInputType keyboardType;
  final Function? onFieldSubmitted;
  final Function? onSaved;
  final Function? validator;
  final bool obscureText;
  final bool visible;
  final FocusNode? focusNode;
  //final bool multiLines;
  final double width;
  final Widget? suffixIcon;
  final TextInputAction textInputAction;
  final TextEditingController? controller;
  final String obscuringCharacter = '•';
  XTextFormFieldLogin(
      {this.label,
      this.hintText,
      this.icon,
      this.controller,
      this.visible = false,
      this.decoration,
      //this.minLines,
      this.suffixIcon,
      this.onChanged,
      this.width = 300,
      //     this.multiLines = false,
      this.keyboardType = TextInputType.text,
      this.textInputAction = TextInputAction.next,
      this.onFieldSubmitted,
      this.onSaved,
      this.initialValue,
      this.validator,
      this.focusNode,
      this.obscureText = false});

  xLabelStyle() {
    return TextStyle(color: Colors.black, fontFamily: 'OpenSans', fontSize: 12);
  }

  xBoxDecorationStyle(Color? color) {
    return BoxDecoration(
      border: Border.all(
        color: color ?? Colors.white,
        width: 1,
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Column(crossAxisAlignment: CrossAxisAlignment.start, children: <Widget>[
      Text(label!, style: xLabelStyle()),
      Container(
        width: width,
        decoration: xBoxDecorationStyle(Colors.white),
        child: TextFormField(
          focusNode: focusNode,
          keyboardType: keyboardType,
          style: TextStyle(color: Colors.white),
          textInputAction: textInputAction,
          onFieldSubmitted: onFieldSubmitted as void Function(String)?,
          obscureText: obscureText,
          onSaved: onSaved as void Function(String?)?,
          obscuringCharacter: obscuringCharacter,
          controller: controller,
          onChanged: onChanged,
          maxLines: 1,
          initialValue: initialValue,
          validator: validator as String? Function(String?)?,
          decoration: InputDecoration(
            border: InputBorder.none,
            prefixIcon: Icon(icon, color: Colors.white),
            hintText: hintText,
            hintStyle: TextStyle(color: Colors.white),
            suffixIcon: Container(width: 20, child: suffixIcon),
          ),
        ),
      ),
    ]);
  }
}

class XBtnbase extends StatelessWidget {
  final String? label;
  final IconData? icon;
  final Function()? onPressed;
  final Decoration? decoration;
  final TextStyle? style;
  final Color? textColor;
  final Color? iconColor;
  final Color color;
  final Widget? child;
  final double? elevation;
  final EdgeInsetsGeometry? padding;
  final OutlinedBorder? shape;
  final double? iconSize;
  final double? iconContainerWidth;
  final double? iconContainerheight;
  final int? typeUI;
  XBtnbase({
    this.label = "",
    this.decoration,
    this.typeUI,
    this.iconContainerWidth,
    this.iconContainerheight,
    this.elevation,
    this.icon,
    this.style,
    this.iconSize,
    this.color = Colors.black,
    this.textColor,
    this.iconColor = Colors.black,
    this.onPressed,
    this.child,
    this.padding,
    this.shape,
  });

  TextStyle styTextBaseWhite([Color? textColor]) {
    return TextStyle(fontSize: 14, color: Colors.white);
  }

  TextStyle styText3() {
    var s = styTextBaseWhite();
    var s1 = TextStyle(fontSize: 13);
    return s.merge(s1);
  }

  TextStyle styText3Black([Color? textColor]) {
    if (textColor == null) textColor = Colors.black;
    var s = styText3();
    var s1 = TextStyle(color: textColor);
    return s.merge(s1);
  }

  Widget selectUI() {
    ////// se 1 il bottone contiene solo l'icona
    if (typeUI == 1) {
      return Container(
        alignment: Alignment.center,
        width: iconContainerWidth,
        height: iconContainerheight,
        // alignment: Alignment.center,
        child: Icon(
          icon,
          color: iconColor,
          size: iconSize,
        ),
      );
      ////// se 2 il bottone contiene solo la label
    } else if (typeUI == 2) {
      return Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [Expanded(child: Text(label ?? "", textAlign: TextAlign.center, style: style ?? styText3Black(textColor)))],
      );
      ////// se 3 il bottone contiene  label + icona
    } else {
      return Row(children: [
        Expanded(
            child: Container(
                child: Icon(
          icon,
          color: iconColor,
          size: iconSize,
        ))),
        SizedBox(
          width: 5,
        ),
        Expanded(flex: 3, child: Text(label ?? "", textAlign: TextAlign.center, style: style ?? styText3Black(textColor)))
      ]);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        decoration: decoration ?? BoxDecoration(border: Border.all(width: 3.4, color: decoration == null ? color : Colors.teal[800]!)),
        child: ElevatedButton(
          style: ElevatedButton.styleFrom(
            elevation: elevation ?? 10,
            padding: EdgeInsets.all(5),
            primary: decoration == null ? color : Colors.transparent,
          ),
          onPressed: onPressed,
          child: selectUI(),
        ));
  }
}

class XTextFormFieldWithLabel extends StatelessWidget {
  final String? label;
  final IconData? icon;
  final TextAlignVertical? textAlignVertical;
  final ValueChanged<String>? onChanged;
  final Function()? onPressed;
  final FocusNode? focusNode;
  final String? initialValue;
  final TextInputType? keyboardType;
  final TextEditingController? controller;
  final Function(String)? onFieldSubmitted;
  final String? hintText;
  VoidCallback? onEditingComplete;
  List<TextInputFormatter>? inputFormatters;
  InputDecoration? decoration = const InputDecoration();
  FormFieldValidator<String>? validator;
  AutovalidateMode? autovalidateMode;
  bool? autovalidate = false;
  GestureTapCallback? onTap;
  bool? expands = false;
  int? maxLines = 1;
  final int? minLines;
  TextAlign? textAlign = TextAlign.start;
  TextAlign? textAlignLabel = TextAlign.start;
  final Alignment? alignment;
  CrossAxisAlignment? crossAxisAlignment = CrossAxisAlignment.center;

  XTextFormFieldWithLabel({
    this.crossAxisAlignment,
    this.label = "",
    this.controller,
    this.textAlign,
    this.alignment,
    this.onEditingComplete,
    this.onTap,
    this.textAlignVertical,
    this.textAlignLabel,
    this.icon,
    this.validator,
    this.autovalidateMode,
    this.onChanged,
    this.autovalidate,
    this.inputFormatters,
    this.decoration,
    this.minLines,
    this.maxLines,
    this.expands = false,
    this.onPressed,
    this.keyboardType,
    this.initialValue = "",
    this.focusNode,
    this.onFieldSubmitted,
    this.hintText = '',
  })  : assert(maxLines == null || maxLines > 0),
        assert(expands != null),
        assert(
          !expands! || (maxLines == null && minLines == null),
          'minLines e MxLines devono essere null quando expands è true.',
        );
  @override
  Widget build(BuildContext context) {
    return Row(
      crossAxisAlignment: crossAxisAlignment ?? CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        SizedBox(width: 5),
        Visibility(
          visible: label != "",
          child: Expanded(
            flex: 1,
            child: Container(
              alignment: alignment,
              child: Text(
                label!,
                style: Theme.of(context).textTheme.caption,
              ),
            ),
          ),
        ),
        Expanded(
          flex: 4,
          child: Container(
            child: FormField(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              builder: (FormFieldState state) {
                return TextFormField(
                  onEditingComplete: onEditingComplete,
                  onTap: onTap,
                  textAlign: textAlign ?? TextAlign.start,
                  controller: controller,
                  validator: validator,
                  autovalidateMode: autovalidateMode ?? AutovalidateMode.always,
                  maxLines: maxLines,
                  initialValue: initialValue,
                  onChanged: onChanged,
                  focusNode: focusNode ?? FocusNode(),
                  keyboardType: keyboardType ?? TextInputType.text,
                  inputFormatters: inputFormatters ?? [],
                  onFieldSubmitted: onFieldSubmitted!,
                  decoration: InputDecoration(
                    hintText: hintText,
                  ),
                );
              },
            ),
          ),
        ),
      ],
    );
  }
}

class XContainerWithLabel extends StatelessWidget {
  final String? label;
  final Color? color;
  final TextStyle? textStyle;
  final double? height;
  final AlignmentGeometry? alignmentContainer;
  final TextAlign? textAlignment;
  final EdgeInsetsGeometry? margin;
  final double? maxwidth;
  XContainerWithLabel(
    this.label, {
    this.height,
    this.alignmentContainer,
    this.textAlignment,
    this.maxwidth,
    this.margin,
    this.color = Colors.white,
    this.textStyle,
  });

  TextStyle styTextBaseBlack([Color? textColor]) {
    if (textColor == null) textColor = Colors.black;
    return TextStyle(fontSize: 14, color: textColor);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin ?? EdgeInsets.only(right: 0),
      height: height,
      color: color,
      width: maxwidth,
      alignment: alignmentContainer ?? Alignment.center,
      child: Text(label ?? " ", style: textStyle ?? styTextBaseBlack(), textAlign: textAlignment ?? TextAlign.center),
    );
  }
}

class XTextFormField extends StatelessWidget {
  final String? label;
  final String hintText;
  final IconData? icon;
  final ValueChanged<String>? onChanged;
  final ValueChanged<String?>? onChangedDropDown;
  final TextInputType keyboardType;
  final Function? onFieldSubmitted;
  final Function? onSaved;
  final Function? onPressed;
  final Function? validator;
  final Function? onEditingComplete;
  final FocusNode? focusNode;
  final bool isPassword;
  final bool multiLines;
  final bool isVisible;
  final bool isVisibleDropDown;
  final bool showSearchBox;
  final double? width;
  final double maxHeight;
  final int? maxLines;
  final double? height;
  final TextInputAction textInputAction;
  final TextEditingController? controller;
  final AutovalidateMode? autovalidateMode;
  final String? initialValue;
  final TextStyle? labelStyle;
  GestureTapCallback? onTap;
  final List<String>? items;
  List<TextInputFormatter>? inputFormatters;
  final Color? textColor;
  final InputBorder? border;
  final InputBorder? disabledBorder;
  final InputBorder? enabledBorder;
  final TextStyle? hintStyle;
  final EdgeInsetsGeometry? contentPadding;
  XTextFormField(
      {this.label,
      this.hintText = '',
      this.icon,
      this.hintStyle,
      this.contentPadding,
      this.border,
      this.disabledBorder,
      this.enabledBorder,
      this.labelStyle,
      this.onEditingComplete,
      this.initialValue,
      this.items,
      this.inputFormatters,
      this.isVisible = false,
      this.isVisibleDropDown = false,
      this.controller,
      this.onPressed,
      this.width,
      this.textColor,
      this.onTap,
      this.maxLines,
      this.height,
      this.maxHeight = 500,
      this.showSearchBox = false,
      this.onChanged,
      this.onChangedDropDown,
      this.multiLines = false,
      this.keyboardType = TextInputType.text,
      this.textInputAction = TextInputAction.next,
      this.onFieldSubmitted,
      this.onSaved,
      this.validator,
      this.focusNode,
      this.autovalidateMode,
      this.isPassword = false});

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Stack(
        alignment: Alignment.bottomRight,
        children: <Widget>[
          Column(children: <Widget>[
            Container(
              width: width,
              height: height,
              child: TextFormField(
                  initialValue: initialValue,
                  controller: controller,
                  onTap: onTap,
                  focusNode: focusNode,
                  keyboardType: (multiLines ? TextInputType.multiline : keyboardType),
                  style: labelStyle ?? TextStyle(color: textColor ?? Colors.white),
                  textInputAction: (multiLines ? TextInputAction.newline : textInputAction),
                  onFieldSubmitted: onFieldSubmitted as void Function(String)?,
                  obscureText: isPassword,
                  onSaved: onSaved as void Function(String?)?,
                  onEditingComplete: onEditingComplete as void Function()?,
                  onChanged: onChanged,
                  inputFormatters: inputFormatters,
                  maxLines: maxLines,
                  autovalidateMode: autovalidateMode,
                  validator: validator as String? Function(String?)?,
                  decoration: InputDecoration(
                    isDense: true,
                    floatingLabelBehavior: FloatingLabelBehavior.always,
                    contentPadding: contentPadding ?? EdgeInsets.only(top: 20, left: 5, bottom: 1),
                    border: border ?? OutlineInputBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)), borderSide: BorderSide(width: 1, color: Colors.white), gapPadding: 3),
                    disabledBorder: disabledBorder ?? OutlineInputBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)), borderSide: BorderSide(width: 1, color: Colors.white), gapPadding: 3),
                    enabledBorder: enabledBorder ?? OutlineInputBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)), borderSide: BorderSide(width: 1, color: Colors.white), gapPadding: 3),
                    labelText: label,
                    labelStyle: labelStyle,
                    hintStyle: hintStyle ??
                        TextStyle(
                          color: textColor ?? Colors.white,
                        ),
                  )),
            ),
            isVisibleDropDown != false
                ? SizedBox(
                    height: 5,
                  )
                : Container(),
          ]),
          isVisible == true
              ? Container(
                  alignment: Alignment.bottomRight,
                  height: 40,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    crossAxisAlignment: CrossAxisAlignment.end,
                    children: <Widget>[
                      Container(
                          padding: EdgeInsets.fromLTRB(0, 0, 0, 5),
                          child: ElevatedButton(
                            style: ElevatedButton.styleFrom(
                              elevation: 1,
                              primary: Colors.transparent,
                            ),
                            onPressed: onPressed as void Function()?,
                            child: Icon(
                              Icons.done,
                              color: Colors.greenAccent,
                            ),
                          )),
                      SizedBox(
                        width: 5,
                      ),
                    ],
                  ),
                )
              : Container()
        ],
      ),
    );
  }
}
